import 'package:contentsquare/contentsquare.dart';
import 'package:contentsquare_example/src/widgets/app_bar.dart';
import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';

class TrackedFlutterWebViewScreen extends StatefulWidget {
  const TrackedFlutterWebViewScreen({Key? key}) : super(key: key);

  @override
  State<TrackedFlutterWebViewScreen> createState() =>
      _TrackedFlutterWebViewScreenState();
}

class _TrackedFlutterWebViewScreenState
    extends State<TrackedFlutterWebViewScreen> {
  final webviewController = WebViewController();

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: appBar(title: 'Tracked (webview_flutter)'),
      body: ContentsquareWebViewTrackerBuilder(
        builder: (context, tracker) {
          return WebViewTrackerInitializer(
            initialUrl:
                'https://tupeuxpastest.csq.io/mobile/web-view-cart.html',
            tracker: tracker,
            webviewController: webviewController,
            child: WebViewWidget(controller: webviewController),
          );
        },
      ),
    );
  }
}

class WebViewTrackerInitializer extends StatefulWidget {
  const WebViewTrackerInitializer({
    Key? key,
    required this.initialUrl,
    required this.tracker,
    required this.webviewController,
    required this.child,
  }) : super(key: key);

  final String initialUrl;
  final ContentsquareWebViewTracker tracker;
  final WebViewController webviewController;

  final Widget child;

  @override
  State<WebViewTrackerInitializer> createState() =>
      _WebViewTrackerInitializerState();
}

class _WebViewTrackerInitializerState extends State<WebViewTrackerInitializer> {
  Future<void> _initTracking() async {
    final tracker = widget.tracker;
    final controller = widget.webviewController;

    // 0. Enable Javascript injection
    await controller.setJavaScriptMode(JavaScriptMode.unrestricted);

    // 1. Register the ContentsquareWebViewController by mapping it to WebViewController
    tracker.registerWebViewController(
      ContentsquareWebViewController(
        evaluateJavascript: (source) async {
          return '${await controller.runJavaScriptReturningResult(source)}';
        },
        currentUrl: controller.currentUrl,
        javascriptChannelCall: (channelName, data) {
          return 'window.$channelName.postMessage($data)';
        },
      ),
    );

    // 2. Register the javascript channels
    for (final javascriptChannel in tracker.javascriptChannels) {
      await controller.addJavaScriptChannel(
        javascriptChannel.name,
        onMessageReceived: (javascriptMessage) {
          javascriptChannel.onMessageReceived(javascriptMessage.message);
        },
      );
    }

    await controller.setNavigationDelegate(
      NavigationDelegate(
        onPageFinished: (_) async {
          // 3. Initialize the tracking every time a new page has finished loading
          //
          // This will be called AFTER the controller is registered and the
          // javascript channels are available since we register the controller
          // and the javascript channels BEFORE loading the initial url.
          await tracker.startPageTracking();
        },
      ),
    );

    await controller.loadRequest(Uri.parse(widget.initialUrl));
  }

  @override
  void initState() {
    super.initState();
    _initTracking();
  }

  @override
  Widget build(BuildContext context) => widget.child;
}
